Equatable

Struct Equatable 

pub struct Equatable<T>(/* private fields */);
A controlled wrapper type that allows for constant time equality checks of a Controlled type. The immediate inner type must also be Controlled (typically Protected).

Examples

Initializing an Equatable:

use vitaminc::protected::{Equatable, Controlled, Protected};
let x: Equatable<Protected<u8>> = 42.into();
let y: Equatable<Protected<u8>> = Equatable::<Protected<u8>>::new(42);

Constant time comparisons

Equatable requires that types are equatable in constant time.

use vitaminc::protected::{Equatable, Protected};
let x: Equatable<Protected<u8>> = 112.into();
let y: Equatable<Protected<u8>> = 112.into();

assert!(x.constant_time_eq(&y));

The Equatable type also implements PartialEq and Eq for easy comparison using the constant time implementation.

use vitaminc::protected::{Equatable, Protected};
let x: Equatable<Protected<u8>> = 112.into();
let y: Equatable<Protected<u8>> = 112.into();
assert_eq!(x, y);

Nesting Equatable types

Constant time comparison also works for nested Equatable types. This way, the ordering or depth of the nesting doesn’t matter, the comparison will always be constant time.

See also crate::Exportable.

use vitaminc::protected::{Exportable, Equatable, Protected};
let x: Equatable<Protected<[u8; 16]>> = [0u8; 16].into();
let y: Exportable<Equatable<Protected<[u8; 16]>>> = Exportable::new([0u8; 16]);

assert_eq!(x, y);

Opaque Debug

Because Equatable wraps Controlled, inner types will never be printed. It’s therefore safe to use it in debug output and in custom types.

use vitaminc::protected::{Equatable, Controlled, Protected};

type Inner = Equatable<Protected<u8>>;

#[derive(Debug, PartialEq)]
struct SafeType(Inner);
let x = SafeType(Inner::new(100));
assert!(format!("{:?}", x).contains("Protected<u8>"));

Usage in a struct

use vitaminc::protected::{Equatable, Protected};

#[derive(Debug, PartialEq)]
struct AuthenticatedString {
  tag: Equatable<Protected<[u8; 32]>>,
  value: String
}

impl AuthenticatedString {
    fn new(tag: [u8; 32], value: String) -> Self {
        Self { tag: tag.into(), value }
    }
}

let a = AuthenticatedString::new([0u8; 32], "Hello, world!".to_string());
let b = AuthenticatedString::new([0u8; 32], "Hello, world!".to_string());
assert_eq!(a, b);

Implementations

Source

impl<T> Equatable<T>

pub fn new(x: <Equatable<T> as Controlled>::Inner) -> Self
where Self: Controlled,

Create a new Equatable from an inner value.

Source

impl<T: Controlled> Equatable<T>

pub fn constant_time_eq(&self, other: &Self) -> bool